@@ -19,8 +19,8 @@ setClass("mi",
                 converged = "logical",
                 coef.mcmc      = "ANY",
                 coef.converged = "logical",
-                preprocess = "logical",
-                mi.info.preprocessed = "ANY",
+                #preprocess = "logical",
+                #mi.info.preprocessed = "ANY",
                 add.noise = "logical"),
             contains  = "list"
 )
@@ -133,3 +133,11 @@ setClass("mi.pooled",
                 mi.fit     = "list"),
                 contains   = "list"
 )
+
+
+setClass("mi.preprocessed",
+            representation(
+                data      = "data.frame" ,
+                mi.info   = "mi.info"),
+            contains  = "list"
+)
